/** 
**************************************************************************
* @file   modbus_data.h
* @brief    Заголовочный файл с описанием даты MODBUS.
* @details  Данный файл необходимо подключается в rs_message.h. После rs_message.h 
* подключается к основному проекту.
* 
* @defgroup MODBUS_DATA
* @ingroup  MODBUS
* @brief    Modbus data description
*
*************************************************************************/

#ifndef _MODBUS_DATA_H_
#define _MODBUS_DATA_H_
  
#include "stdint.h"
//--------------DEFINES FOR REGISTERS---------------
// DEFINES FOR ARRAYS
/** 
  * @addtogroup MODBUS_DATA_RERISTERS_DEFINES
  * @ingroup    MODBUS_DATA
  * @brief      Defines for registers
  Структура дефайна адресов
  @verbatim    
  Для массивов регистров:
    R_<NAME_ARRAY>_ADDR   -  модбас адресс первого регистра в массиве
    R_<NAME_ARRAY>_QNT    -  количество регистров в массиве
  
  При добавлении новых массивов регистров, необходимо их добавить в функцию MB_DefineRegistersAddress

		if(MB_Check_Address_For_Arr(Addr, Qnt, R_<NEW_ARRAY>_ADDR, R_<NEW_ARRAY>_QNT) == NO_ERRORS)
		{
			*pRegs = MB_Set_Register_Ptr(&<NEW_ARRAY>, Addr);	// начало регистров хранения/входных
		}
  @endverbatim
  * @{
  */
  
  
/** 
  * @brief Регистры хранения
  */
	
	
	#define MAX_SENSE 30
	
typedef struct //MB_DataInRegsTypeDef
{
	uint16_t sens_Temp[MAX_SENSE];
  uint16_t UzptPLUS;
	uint16_t UzptMINUS;
	uint16_t UzptPLUS_MIN;
	

	
	
}MB_DataInRegsTypeDef;


/** 
  * @brief Входные регистры 
  */
typedef struct //MB_DataInRegsTypeDef
{
  uint16_t set_Temp[MAX_SENSE];
}MB_DataHoldRegsTypeDef;


// DEFINES FOR INPUT REGISTERS ARRAYS
#define   R_INPUT_ADDR                 0
#define   R_INPUT_QNT                  10

// DEFINES FOR HOLDING REGISTERS ARRAYS
#define   R_HOLDING_ADDR                   0
#define   R_HOLDING_QNT                    10

 
/** MODBUS_DATA_RERISTERS_DEFINES
  * @} 
  */

//----------------DEFINES FOR COILS-----------------
/** 
  * @addtogroup MODBUS_DATA_COILS_DEFINES
  * @ingroup    MODBUS_DATA
  * @brief      Defines for coils
  @verbatim    
  Структура дефайна
  Для массивов коилов:
    C_<NAME_ARRAY>_ADDR   - модбас адресс первого коила в массиве
    C_<NAME_ARRAY>_QNT    - количество коилов в массиве (минимум 16)

  При добавлении новых массивов коилов, необходимо их добавить в функцию MB_DefineCoilsAddress

    if(MB_Check_Address_For_Arr(Addr, Qnt, C_<NEW_ARRAY>_ADDR, C_<NEW_ARRAY>_QNT) == NO_ERRORS)
    {
      *pCoils = MB_Set_Coil_Reg_Ptr(&<NEW_ARRAY>, Addr);
    }

  @endverbatim
  * @{
  */
  
  
  
/** 
  * @brief     Коилы
  * @details   Желательно с помощью reserved делать стркутуру кратной 16-битам
  */
typedef union 
	{
		struct {
			unsigned state_val_01:1;
			unsigned state_val_02:1;
			unsigned state_val_03:1;
			unsigned state_val_04:1;
			unsigned state_val_05:1;
			unsigned state_val_06:1;
			unsigned state_val_07:1;
			unsigned state_val_08:1;
			unsigned state_val_09:1;
			unsigned state_val_10:1;
			unsigned state_val_11:1;
			unsigned state_val_12:1;
			unsigned state_val_13:1;
			unsigned state_val_14:1;
			unsigned state_val_15:1;
			unsigned state_val_16:1;
		} state_val_bit;
		uint16_t all;
		
		
	}word;
typedef struct //MB_DataCoilsTypeDef
{
	
	
	word coils;
	unsigned init_param:1;
  unsigned reserved:15;
	
}MB_DataCoilsTypeDef;

// DEFINES FOR COIL ARRAYS
#define C_CONTROL_ADDR                0
#define C_CONTROL_QNT                 16

/** MODBUS_DATA_COILS_DEFINES
  * @} 
  */
  
  
//-----------MODBUS DEVICE DATA SETTING-------------
// MODBUS DATA STRUCTTURE
/** 
  * @brief Структура со всеми регистрами и коилами модбас 
  * @ingroup  MODBUS_DATA
  */
typedef struct // tester modbus data
{
  MB_DataInRegsTypeDef      InRegs;     ///< Modbus input registers @ref MB_DataInRegsTypeDef

  MB_DataCoilsTypeDef       Coils;      ///< Modbus coils @ref MB_DataCoilsTypeDef
  
  MB_DataHoldRegsTypeDef    HoldRegs;   ///< Modbus holding registers @ref MB_DataHoldRegsTypeDef
}MB_DataStructureTypeDef;
extern MB_DataStructureTypeDef MB_DATA;


/** 
  * @brief Структура для объекта Modbus 
  * @ingroup  MODBUS_DATA
  */
typedef struct
{
  unsigned  length;
  char      *name;
}MB_DeviceObjectTypeDef;
/** 
  * @brief Структура для объектов Modbus
  * @ingroup  MODBUS_DATA
  */
typedef struct
{
  MB_DeviceObjectTypeDef VendorName;
  MB_DeviceObjectTypeDef ProductCode;
  MB_DeviceObjectTypeDef Revision;
  MB_DeviceObjectTypeDef VendorUrl;
  MB_DeviceObjectTypeDef ProductName;
  MB_DeviceObjectTypeDef ModelName;
  MB_DeviceObjectTypeDef UserApplicationName;
}MB_DeviceIdentificationTypeDef;
void MB_DevoceInentificationInit(void);


#endif //_MODBUS_DATA_H_

/////////////////////////////////////////////////////////////
///////////////////////TEMP/OUTDATE/OTHER////////////////////